--|  
--| Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/ascl-debugging_support.ads,v $
--| Description     : Debuging object
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/23 09:48:34 $
--| Status          : $State: Released $
--|
--|
--| Functional Description
--| ======================
--| This packages provides a so called debugging object. An application
--| has to provide the output procedure.
--| The following procedures are provided:
--|
--| Enter/Leave - Enter/Leave a context
--| Trace       - Output trace information
--|
--|
--|
--| Component Data
--| ==============
--| Debugging Level  - Set/Get the following values are possible:
--|
--|                    Quiet   - Not trace at all
--|                    Logic   - Only the call trace is shown
--|                    Verbose - Every trace information is shown
--|
--|
--| Error Handling
--| ==============
--|
--|
--| Extension
--| =========
--|
--| Restrictions
--| ============
--| Tasking: no
--| Y2K    : yes
--|
--|
--| History
--| =======
--| $Log: ascl-debugging_support.ads,v $
--| Revision 1.2  2000/01/23 09:48:34  ACDE
--| Release Version base_1_1
--|
--| Revision 1.1  2000/01/23 09:46:34  ACDE
--| Release Version 1_0
--|
--| Revision 1.13  1999/11/27 16:56:18  ACDE
--| Release Version 1P010
--|
--| Revision 1.12  1999/11/27 16:50:10  ACDE
--| Preinstallation checkin
--|
--| Revision 1.11  1999/11/27 16:45:46  ACDE
--| Release Version 1P009
--|
--| Revision 1.10  1999/11/27 16:44:56  ACDE
--| Preinstallation checkin
--|
--| Revision 1.9  1999/11/27 16:42:27  ACDE
--| Preinstallation checkin
--|
--| Revision 1.8  1999/11/27 16:38:33  ACDE
--| Preinstallation checkin
--|
--| Revision 1.7  1999/11/27 16:35:39  ACDE
--| Preinstallation checkin
--|
--| Revision 1.6  1999/11/27 16:33:28  ACDE
--| Preinstallation checkin
--|
--| Revision 1.5  1999/11/27 16:33:00  ACDE
--| Preinstallation checkin
--|
--| Revision 1.4  1999/11/27 16:30:28  ACDE
--| Preinstallation checkin
--|
--| Revision 1.3  1999/11/27 15:47:05  ACDE
--| Installation checkin
--|
--| Revision 1.2  1999/11/27 09:33:55  ACDE
--| sg
--|
--| Revision 1.1  1999/11/26 14:38:07  ACDE
--| error corrections and cleanup
--|
--| Revision 1.2  1999/08/16 18:17:48  erdmann
--| Documentation added, Test complete
--|
--| Revision 1.1  1999/08/01 19:14:59  erdmann
--| Specification complete and compiled
--|
--|
package ASCL.Debugging_Support is
   
   ---=====================================================================---
   ---===             C O M P O N E N T   I N T E R F A C E             ===---
   ---=====================================================================---

   type Object is abstract tagged private;
   type Handle is access all Object'Class;

   procedure Initialize( this : in out Object'Class );
   procedure Finalize( this : in out Object'Class );

   ---=====================================================================---
   ---===                  A T T R I B U T E S                       ===---
   ---=====================================================================---
   type Debug_Level is ( Quiet, Logic, Verbose );

   procedure Set( this : in out Object'Class; level : in Debug_Level );
   function  Get( this : in Object'Class ) return Debug_Level;

   ---=====================================================================---
   ---===                     M E T H O D S                             ===---
   ---=====================================================================---

   ---------------------------------------------------------------------------
   --| Description    : This method indicates that an error has occured. 
   --| Preconditions  : Instance has been initialized.
   --| Postconditions : -
   --| Exceptions     : -
   --| Note           : Ouput is used to output the error message.
   ---------------------------------------------------------------------------
   procedure Error( this : in Handle; text : in String );

   ---------------------------------------------------------------------------
   --| Description    : Output trace information 
   --| Preconditions  : Instance has been initialized and trace level
   --|                  is Verbose.
   --| Postconditions : -
   --| Exceptions     : -
   --| Note           : Ouput is used to output the error message.
   ---------------------------------------------------------------------------
   procedure Trace( this : in Handle; text : in String );

   ---------------------------------------------------------------------------
   --| Description    : Enter a logical block procedure or function 
   --| Preconditions  : Instance has been initialized and trace level
   --|                  is Verbose or Logic.
   --| Postconditions : -
   --| Exceptions     : -
   --| Note           : Output is used for trace information
   ---------------------------------------------------------------------------
   procedure Enter( this : in Handle; text : in String );

   ---------------------------------------------------------------------------
   --| Description    : Leave a logical block, procedure o function
   --| Preconditions  : Instance has been initialized an trace level
   --|                  is Verbose or Logic.
   --| Postconditions :
   --| Exceptions     :
   --| Note           : Output is used for trace information
   ---------------------------------------------------------------------------
   procedure Leave( this : in Handle; text : in String );

   ---=====================================================================---
   ---===                   E X T E N S I O N                           ===---
   ---=====================================================================---  

   procedure Output( this : in out Object; text : in String ) is abstract;

   ---=====================================================================---
private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   type Object is abstract tagged record
         data : Object_Data_Access := null;
      end record; 

end ASCL.Debugging_Support;


