--|
--| Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/ascl-dt-container.adb,v $
--| Description     : User Object
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/23 09:48:34 $
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--|
--|
--| Restrictions
--| ============
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: ascl-dt-container.adb,v $
--| Revision 1.2  2000/01/23 09:48:34  ACDE
--| Release Version base_1_1
--|
--| Revision 1.1  2000/01/23 09:46:34  ACDE
--| Release Version 1_0
--|
--| Revision 1.12  1999/11/27 16:56:18  ACDE
--| Release Version 1P010
--|
--| Revision 1.11  1999/11/27 16:50:10  ACDE
--| Preinstallation checkin
--|
--| Revision 1.10  1999/11/27 16:45:46  ACDE
--| Release Version 1P009
--|
--| Revision 1.9  1999/11/27 16:44:56  ACDE
--| Preinstallation checkin
--|
--| Revision 1.8  1999/11/27 16:42:27  ACDE
--| Preinstallation checkin
--|
--| Revision 1.7  1999/11/27 16:38:33  ACDE
--| Preinstallation checkin
--|
--| Revision 1.6  1999/11/27 16:35:39  ACDE
--| Preinstallation checkin
--|
--| Revision 1.5  1999/11/27 16:33:28  ACDE
--| Preinstallation checkin
--|
--| Revision 1.4  1999/11/27 16:33:00  ACDE
--| Preinstallation checkin
--|
--| Revision 1.3  1999/11/27 16:30:28  ACDE
--| Preinstallation checkin
--|
--| Revision 1.2  1999/11/27 15:47:05  ACDE
--| Installation checkin
--|
--| Revision 1.1  1999/11/27 09:33:55  ACDE
--| sg
--|
--|
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Unchecked_Deallocation;

with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
use  ASCL;

with ASCL.DT.Container_Element;    use ASCL.DT.Container_Element;
use  ASCL.DT;

package body ASCL.DT.Container is

   Version : constant String := "$Id: ascl-dt-container.adb,v 1.2 2000/01/23 09:48:34 ACDE Released ACDE $";
   Module  : constant String := "ascl-dt-container.adb";

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---
   Bucket_Size    : constant Positive := 1000;
   Nbr_of_Buckets : constant Positive := 1000;

   type OT_Index      is new Integer range 0..Bucket_Size-1;
   type Bucket_Number is new Integer range 0..Nbr_Of_Buckets-1;
    
   --- The object table and the bucket it self
   type Object_Table  is array( OT_Index ) of Container_Element.Handle; 
   type Bucket_Access is access Object_Table;

   --|
   --| This is the instance of the component internal data.
   --|

   type Bucket_Array is array(Bucket_Number) of Bucket_Access;

   type Object_Data is record
         debug   : Debugging_Support.Handle := null;
	 Buckets : Bucket_Array := (others => null );
      end record;



   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object'Class;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| This procedure is used to access the instance data of a
   --| container.
   --|
   function Access_Object_Data ( this : in Object'Class; info : in String ) 
                                                  return Object_Data_Access is
   begin
      if this.Data = null then
         raise Not_Initialized;
      end if;

      return this.Data;
   exception 
      when The_Error : Others  =>
         Error( this, The_Error, info );
	 raise;
   end Access_Object_Data;


   --|
   --| The following three procedures are used to map between reference
   --| and bucket number & ot index.
   --|
   function To_Reference( b : Bucket_Number;  O : OT_Index ) return Reference is
   begin
      return Integer(b)*Bucket_Size + Integer(o);
   end To_Reference;

   function To_Bucket_Number( r : Reference ) return Bucket_Number is
   begin
       return Bucket_Number(r / Bucket_Size);
   end To_Bucket_Number; 

   function To_OT_Index( r : Reference ) return OT_Index is
   begin
      return OT_Index( r mod Bucket_Size );
   end To_OT_Index;

   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Intialize the Object data and call the Initalization
   --| procedure of the extention.
   --|
   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data  : Object_Data_Access renames this.data;
   begin
      Enter( debug, Module & ".Initialize" );
      if Data /= null then
         raise Usage_Error;
      end if;

      Data := new Object_Data;

      Data.Debug := debug; 
       
      Leave( debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extention.
   --|
   procedure Finalize( this : in out Object'Class ) is
      data   : Object_Data_Access renames this.data;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      if data = null then
         return;
      end if;

      Free( data );

      this.data := null;

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Purpose:
   --|
   --| Pre-Conditions:
   --|
   --| Post-Conditions:
   --|
   --|
   function Add( this : in Object'Class; item : Container_Element.Handle ) 
                              return Reference is
      Data   : Object_Data_Access := Access_Object_Data( this, ".Add");
      Debug  : Debugging_Support.Handle := null;
      result : Reference := 0;
   begin
      Enter( Debug, ".Add");
   
      Search_Loop:
	      for bidx in Bucket_Number loop
	         if Data.Buckets(bidx) /= null then
		    declare
		       aBucket : Bucket_Access := Data.Buckets(bidx);
		    begin
		       for oidx in OT_Index loop
		          if aBucket(oidx) = null then
			     aBucket(oidx) := item;

			     Result := To_Reference(bidx, oidx);
			     exit Search_Loop;
		          end if;
		       end loop;
		    end;
		 end if;
              end loop Search_Loop;

   
      if Result = 0 then
         for i in Bucket_Number loop
	    if Data.Buckets(i) = null then
	       Data.Buckets(i) := new Object_Table'( (others=>null) );   --'
	       Result := Add( this, item );
	    end if;
	 end loop; 
      end if;
      if Result = 0 then
         null;
      end if;
       
      Leave( Debug,"element stored reference=" & Reference'Image(Result));
      return Result;
   end Add;

end ASCL.DT.Container;

