--|                              -*- Mode: Ada -*-
--| Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/ascl-dt-index_tree.ads,v $
--| Description     : Index Tree
--| Author          : Michael Erdmann
--| Created On      : 27.6.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/23 09:48:34 $
--| Update Count    : 0
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--| This class represents an index tree. An index tree is used
--| to store information addressed by an index key. 
--| This information is an implementation of the tree_element class.
--| Each instance of the tree_element is identified by 
--| a key. The implementation of the tree_element has to supply
--| a attribute function which returns of every position in 
--| in the key an index.  
--|
--|
--| History
--| =======			       
--| $Log: ascl-dt-index_tree.ads,v $
--| Revision 1.2  2000/01/23 09:48:34  ACDE
--| Release Version base_1_1
--|
--| Revision 1.1  2000/01/23 09:46:34  ACDE
--| Release Version 1_0
--|
--| Revision 1.7  1999/11/27 16:56:18  ACDE
--| Release Version 1P010
--|
--|
--| Revision 1.1  1999/11/27 16:39:17  ACDE
--| No comments
--|
--| Revision 1.1  1999/12/23 06:59:53  ACDE
--| No comments
--|
--| Revision 1.1  1999/07/19 19:30:19  erdmann
--| No comments
--|
--|
--|
with ASCL.DT.Tree_Element;      use ASCL;

package Ascl.DT.Index_Tree is
   
   ---====================================================================---
   ---===         C O M P O N E N T    I N T E R F A C E               ===---
   ---====================================================================---
   type Object( size : Integer ) is abstract tagged private;
   type Handle is access Object'Class;


   procedure Initialize( this : in out Object'Class );
   procedure Finalize( this : in out Object'Class );

   ---=====================================================================---
   ---===                    E X C E P T I O N S                        ===---
   ---=====================================================================---

   ---====================================================================---
   ---===            A T T R I B U T E    F U N C T I O N S            ===---
   ---====================================================================---
   function Width( this : in Object'Class ) return Integer;

   ---====================================================================---
   ---===                      M E T H O D S                           ===---
   ---====================================================================---
   procedure Add( this : in out Object'Class; item : Tree_Element.Handle );
   procedure Process( this : in out Object'Class );

   type Selection is ( First, Next );

   function Get( this : in Object; mode : Selection := Next )
                           return Tree_Element.Handle;

 
   procedure Load( this : in out Object'Class; fileName : in String );
   procedure Save( this : in out Object'Class; fileName : in String );
   ---====================================================================---
   ---===                     E X T E N S I O N S                      ===---
   ---====================================================================---
   procedure Action( this  : in out Object; 
                     item  : in Tree_Element.Handle; 
                     level : in Integer)    is abstract;

 
private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   function Initialize( size : Integer ) return Object_Data_Access;

   type Object( size : Integer ) is abstract tagged record
         data        : Object_Data_Access := Initialize( size );
      end record; 

end Ascl.DT.Index_Tree;


