--|  
--| Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/ascl-dt-list.ads,v $
--| Description     : Simple List Package
--| Author          : Michael Erdmann
--| Created On      : 23.11.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/23 09:48:34 $
--| Status          : $State: Released $
--|
--|
--| Functional Description
--| ======================
--| This package provides a basic linked list. The generic package 
--| takes the type of the elements as input.
--|
--|
--| Component Data
--| ==============
--|
--|
--| Operations on the component data
--| ================================
--|
--| Extension
--| =========
--|
--| Restrictions
--| ============
--| Tasking : No
--|
--| History
--| =======
--| $Log: ascl-dt-list.ads,v $
--| Revision 1.2  2000/01/23 09:48:34  ACDE
--| Release Version base_1_1
--|
--| Revision 1.1  2000/01/23 09:46:34  ACDE
--| Release Version 1_0
--|
--| Revision 1.11  1999/11/27 16:56:18  ACDE
--| Release Version 1P010
--|
--|

with ASCL.Debugging_Support;
use  ASCL;

generic 

   type Item_Type is private;
    
package ASCL.DT.List is
   
   ---=====================================================================---
   ---===             C O M P O N E N T   I N T E R F A C E             ===---
   ---=====================================================================---
   type Object is tagged private;
   type Handle is access Object'Class;

   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null);

   procedure Finalize( this : in out Object'Class );

   ---=====================================================================---
   ---===                  A T T R I B U T E S                       ===---
   ---=====================================================================---
    ---=====================================================================---
   ---===                     M E T H O D S                             ===---
   ---=====================================================================---

   ---------------------------------------------------------------------------
   --| Description    :
   --|    Append the element to the end of the list
   --| Preconditions  :
   --|    1. Initialized previously
   --| Postconditions :
   --|    none
   --| Exceptions     :
   --|    none
   --| Note           :
   --|    none
   ---------------------------------------------------------------------------
   procedure Append( this : in out Object'Class; item  : in Item_Type );

   type List_handler is access 
      function( this : in Object'Class; item : Item_Type ) return Boolean;
                                           
   procedure Perform( this : in out Object'Class; handler : in List_Handler );
   ---=====================================================================---
   ---===                   E X T E N S I O N                           ===---
   ---=====================================================================---

   ---=====================================================================---
private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   type Object is  tagged record
         data : Object_Data_Access := null;
      end record; 

end ASCL.DT.List;


