-- 
-- Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/ascl-ob-memory_pool.ads,v $
-- Description     : Memory Pool
-- Author          : Michael Erdmann
-- Created On      : Sun Jul 26 08:24:57 1998
-- Last Modified By: $Author: ACDE $
-- Last Modified On: $Date: 2000/01/23 09:48:34 $
-- Status          : $State: Released $
--
-- Functional Description
-- ======================
-- This class groups all object together in orsder to allow loading
-- and writing of objects.
-- 
-- Restrictions
-- ============
--
--
-- History
-- =======
--
--
-- History
-- =======
-- $Log: ascl-ob-memory_pool.ads,v $
-- Revision 1.2  2000/01/23 09:48:34  ACDE
-- Release Version base_1_1
--
-- Revision 1.1  2000/01/23 09:46:34  ACDE
-- Release Version 1_0
--
-- Revision 1.11  1999/11/27 16:56:18  ACDE
-- Release Version 1P010
--
-- Revision 1.2  1999/11/27 15:47:05  ACDE
-- Installation checkin
--
-- Revision 1.1  1999/11/27 09:33:55  ACDE
-- sg
--
--
with System;                      use System;
with System.Storage_Pools;        use System.Storage_Pools;
with System.Storage_Elements;     use System.Storage_Elements;

with ASCL.Debugging_Support;      
use  ASCL;

package ASCL.OB.Memory_Pool is
   ---==========================================================================---
   ---===            C O M P O N E N T    I N T E R F A C E                  ===---
   ---==========================================================================---

   type Object( Size : Storage_Count ) is 
      new Root_Storage_Pool with private;

   procedure Allocate( Pool            : in out Object;
                       Storage_Address : out    Address;
                       Size            : in     Storage_Count;
                       Alignment       : in     Storage_Count );

   procedure Deallocate( Pool          : in out Object;
                       Storage_Address : in     Address;
                       Size            : in     Storage_Count;
                       Alignment       : in     Storage_Count );

   function Storage_Size ( pool : Object ) return Storage_Count;

   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null );

   procedure Finalize( this : in out Object );


   Memory_Allocation_Failure : exception;
   To_Many_Objects           : exception;
   Pool_Exceeded             : exception;

   ---==========================================================================---
   ---===                     A T T R I B U T E S                            ===---
   ---==========================================================================---
   function File_Name( this : in Object) return String;
   procedure File_Name( this : in out Object; name : in String );

   ---==========================================================================---
   ---===                      M E T H O D S                                 ===---
   ---==========================================================================--- 
   procedure Store( this : in Object );
   procedure Load( this : in Object );

   ---==========================================================================---
private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   type Object( Size : Storage_Count ) is new Root_Storage_Pool with
      record
         OT_Size         : Storage_Count      := Size;
         data            : Object_Data_Access := null;
      end record;
  
end ASCL.OB.Memory_Pool;

