--|                              -*- Mode: Ada -*-
--| Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/ascl-rt-event_q.adb,v $
--| Description     : Event Qeue
--| Author          : Michael Erdmann
--| Created On      : 27.7.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/23 09:48:34 $
--| Update Count    : 0
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--| This package maintains the data for an event queue.
--|
--|
--| History
--| =======
--| $Log: ascl-rt-event_q.adb,v $
--| Revision 1.2  2000/01/23 09:48:34  ACDE
--| Release Version base_1_1
--|
--| Revision 1.1  2000/01/23 09:46:34  ACDE
--| Release Version 1_0
--|
--| Revision 1.11  1999/11/27 16:56:18  ACDE
--| Release Version 1P010
--|
--| Revision 1.2  1999/11/27 15:47:05  ACDE
--| Installation checkin
--|
--| Revision 1.1  1999/11/27 09:33:55  ACDE
--| sg
--|
--|
with ASCL.RT.Event;               use ASCL.RT.Event;

package body ASCL.RT.Event_Q is

   --=====================================================================--
   --==                 L O C A L   D A T A                             ==--
   --=====================================================================--

   type Q_Element;
   type Q_Element_Ptr is access Q_Element;

   type Q_Element is record
         next : Q_Element_Ptr := null;
         ev   : Event.Handle  := Event.Null_Event;
      end record;

   protected type Object_Data is

         procedure Put( ev : Event.Handle );
         procedure Defere( ev : Event.Handle );
         entry     Get( ev : out Event.Handle );

      private
         first         : Q_Element_Ptr := null;
         last          : Q_Element_Ptr := null;

         first_defered : Q_Element_Ptr := null;
         last_defered  : Q_Element_Ptr := null;
      end Object_Data;

   --=====================================================================--
   --==               INITIALIZE COMPONENT                              ==--
   --=====================================================================--

   procedure Initialize( this  : in out Object;
   			 debug : in Debugging_Support.Handle := null ) is
   begin
      null;
   end Initialize;

   procedure Finalize( this : in out Object ) is
   begin
      null;
   end Finalize;
   --=====================================================================--
   --==            A T T R I B U T E    M E T H O D S                   ==--
   --=====================================================================--


   --=====================================================================--
   --==                        M E T H O D S                            ==--
   --=====================================================================--

   --|
   --| protected access methods for the que data is defined
   --| here.
   --|
   protected body Object_Data is

      procedure Put( ev : in Event.Handle ) is
         element : Q_Element_Ptr := new Q_Element;
      begin
         element.ev := ev;

         if last = null then
            first := element;
         else
            last.next := element;
         end if;
         last := element;
      end Put;

      procedure Defere ( ev : in Event.Handle ) is
         element : Q_Element_Ptr := new Q_Element;
      begin
         element.ev := ev;

         if last_defered = null then
            first_defered := element;
         else
            last_defered.next := element;
         end if;
         last_defered := element;
      end Defere;

      entry Get( ev : out Event.Handle ) when first /= null is
      begin
         if first_defered = null then
            ev := first.ev;
            first := first.next;
         else
            ev := first_defered.ev;
            first_defered := first_defered.next;
         end if;
      end Get;

   end Object_Data;


   --|
   --| Append an event to the end of the event que
   --|
   procedure Put( this : in Object'Class; evt : in Event.Handle ) is
      data : Object_Data_Access := this.data;
   begin
      if data = null or evt = Event.Null_Event then
         raise Usage_Error;
      end if;

      data.Put( evt );
   end Put;

   --|
   --| Read event element form the begining of the que.
   --|
   function Get( this : in Object'Class ) return Event.Handle is
      Data   : Object_Data_Access := this.data;
      Result : Event.Handle := Event.Null_Event;
   begin
      if Data = null then
         raise Not_Initialized;
      end if;

      Data.Get( Result );
      return Result;
   end Get;

   --|
   --|
   --|
   procedure Defere( this : in out Object'Class; evt : in Event.Handle ) is
      Data : Object_Data_Access := this.data;
   begin
      if data = null or evt = Event.Null_Event then
         raise Usage_Error;
      end if;

      data.Defere( evt );

   exception
      when The_Error : Others =>
         raise;         
   end Defere;

end ASCL.RT.Event_Q;
