--|                              -*- Mode: Ada -*-
--| Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/ascl-rt-fsm.ads,v $
--| Description     : Template Specification
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/23 09:48:34 $
--| Update Count    : 0
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--|
--| A FSM (finite state machine) is the basic workhouse of 
--| realtime systems. All FSM's are registerd and monitored for 
--| state changes. If an FSM does not change its state for
--| a cerain tine, the instance is assumed to be dead.
--|
--| An FSM instance consits of an supervisor task and 
--| a number of application tasks which are monitored
--| by the supervisor. 
--| The application task is assumed to be build from 
--| states. 
--|
--| Component Data
--| ==============
--|
--|
--| Operations on the component data
--| ================================
--|
--| Initialize
--| Finalize
--|
--| Error Handling
--| ==============
--|
--|
--| Extension
--| =========
--|
--|
--| History
--| =======
--| $Log: ascl-rt-fsm.ads,v $
--| Revision 1.2  2000/01/23 09:48:34  ACDE
--| Release Version base_1_1
--|
--| Revision 1.1  2000/01/23 09:46:34  ACDE
--| Release Version 1_0
--|
--| Revision 1.11  1999/11/27 16:56:18  ACDE
--| Release Version 1P010
--|
--| Revision 1.1  1999/11/27 09:33:55  ACDE
--| sg
--|
--|
with ASCL.RT.State;
with ASCL.Debugging_Support;
use  ASCL;

package ASCL.RT.Fsm is
   
   ---=====================================================================---
   ---===             C O M P O N E N T   I N T E R F A C E             ===---
   ---=====================================================================---
   type Object is abstract tagged private;
   type Handle is access Object'Class;

   procedure Initialize( this : in out Object'Class;
                         debug : in Debugging_Support.Handle := null );
   procedure Finalize( this : in out Object'Class );
   ---=====================================================================---
   ---===                  A T T R I B U T E S                       ===---
   ---=====================================================================---

   ---=====================================================================---
   ---===                     M E T H O D S                             ===---
   ---=====================================================================---
   procedure Register( this    : in out Object'Class;
                       aState  : in State.Handle;
                       stateID : in State.State_ID );

   procedure Initial( this : in out Object'Class; aState : in State.State_ID);
   procedure Final( this : in out Object'Class;   aState : in State.State_ID);

   ---=====================================================================---
   ---===                   E X T E N S I O N                           ===---
   ---=====================================================================---
   procedure FSM_Init( this : in out Object ) is abstract;

   ---=====================================================================---
private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   type Object is abstract tagged record
         data : Object_Data_Access := null;
      end record;


end ASCL.RT.Fsm;

