--                              -*- Mode: Ada -*-
-- Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/ascl-sv-agent.ads,v $
-- Description     : agent interface
-- Author          : Michael Erdmann
-- Created On      : Sun Jul 26 08:24:57 1998
-- Last Modified By: $Author: ACDE $
-- Last Modified On: $Date: 2000/01/23 09:48:34 $
-- Update Count    : 0
-- Status          : $State: Released $
--
-- Functional Description
-- ======================
-- This is the abstract interface to the agent implementation. An
-- agent implements the processing of the client data and returns
-- data to the client.
--
-- Initialize   -- Initialize the agent data
-- Finalize     -- finalize the agent data
--
-- Process      -- takes as input a strings which has been recevied
--                 from the client.
--                 The funtion returs a string as a response to the 
--                 input.
--
-- History
-- =======
-- $Log: ascl-sv-agent.ads,v $
-- Revision 1.2  2000/01/23 09:48:34  ACDE
-- Release Version base_1_1
--
-- Revision 1.1  2000/01/23 09:46:34  ACDE
-- Release Version 1_0
--
-- Revision 1.11  1999/11/27 16:56:18  ACDE
-- Release Version 1P010
--
-- Revision 1.1  1999/11/27 09:33:55  ACDE
-- sg
--
-- Revision 1.2  1999/09/26 18:17:46  erdmann
-- Put/Get added and interface to process changed
--
-- Revision 1.1  1999/09/25 19:37:21  erdmann
-- No comments
--
--
--
--
package ASCL.SV.Agent is

   type Object is abstract tagged private;
   type Reference is access all Object'Class;

   procedure Initialize( O : in out Object) is abstract;
   procedure Finalize( O : in out Object ) is abstract;
   
   procedure Process( O    : in out Object; 
                      Data : in String ) is abstract;

   procedure Put( This : in out Object'Class; toSend : in String );
   function  Get( This : in Object'Class ) return String;
   function  Data_Available( This : in Object'Class ) return Boolean;

   End_Of_Processing : exception;

private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   function Initialize return Object_Data_Access;

   type Object is abstract tagged record
         data : Object_Data_Access := Initialize;
      end record;

end ASCL.SV.Agent;
