--                              -*- Mode: Ada -*-
-- Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/ascl-sv-server.ads,v $
-- Description     : server component
-- Author          : Michael Erdmann
-- Created On      : Sat Jul 25 19:11:10 1998
-- Last Modified By: $Author: ACDE $
-- Last Modified On: $Date: 2000/01/23 09:48:34 $
-- Status          : $State: Released $
--
-- Functional Description
-- ======================
-- This is a server object. The server waits for scoket connections 
-- on a specified port. The port number may by given as the second
-- parameter of the initialize call. The server will create instances
-- of the Agent-Datatype for every client/server relation by means of 
-- the create method. As a result a server instance supports only
-- a single agent type.
--
--
-- History
-- =======
-- $Log: ascl-sv-server.ads,v $
-- Revision 1.2  2000/01/23 09:48:34  ACDE
-- Release Version base_1_1
--
-- Revision 1.1  2000/01/23 09:46:34  ACDE
-- Release Version 1_0
--
-- Revision 1.11  1999/11/27 16:56:18  ACDE
-- Release Version 1P010
--
-- Revision 1.10  1999/11/27 16:50:10  ACDE
-- Preinstallation checkin
--
-- Revision 1.9  1999/11/27 16:45:46  ACDE
-- Release Version 1P009
--
-- Revision 1.8  1999/11/27 16:44:56  ACDE
-- Preinstallation checkin
--
-- Revision 1.7  1999/11/27 16:42:27  ACDE
-- Preinstallation checkin
--
-- Revision 1.6  1999/11/27 16:40:22  ACDE
-- Preinstallation checkin
--
-- Revision 1.5  1999/11/27 16:33:28  ACDE
-- Preinstallation checkin
--
-- Revision 1.4  1999/11/27 16:33:00  ACDE
-- Preinstallation checkin
--
-- Revision 1.3  1999/11/27 16:30:28  ACDE
-- Preinstallation checkin
--
-- Revision 1.2  1999/11/27 15:47:05  ACDE
-- Installation checkin
--
-- Revision 1.1  1999/11/27 09:33:55  ACDE
-- sg
--
-- Revision 1.3  1999/09/28 07:40:22  erdmann
-- Debuging Support added
--
-- Revision 1.2  1999/09/26 18:18:53  erdmann
-- Agent interface changed (Put/Get)
--
-- Revision 1.1  1999/09/25 19:37:32  erdmann
-- No comments
--
with ASCL.SV.Agent;   
with ASCL.Debugging_Support;   use ASCL;

package ASCL.SV.Server is

   type Object( Port : Positive ) is abstract tagged private;
   type Reference is access all Object'Class; 	        	

   --=======================================================================--
   --===                    COMPONENT INTERFACE                          ===--
   --=======================================================================--
   procedure Initialize( O     : in Reference; 
                         Port  : in Integer := 0;
                         debug : in Debugging_Support.Handle := null );

   procedure Finalize( O : in Reference );

   --=======================================================================--
   --===                        EXTENSIONS                               ===--
   --=======================================================================--

   function  Create( O : in Object ) return Agent.Reference is abstract;
   procedure Delete( A : Agent.Object'Class ) is abstract;   

private

   type Object_Data;
   type Object_Data_Access is access Object_Data;

   function Initialize( Port : in Positive ) return Object_Data_Access;

   type Object( Port : Positive ) is abstract tagged record
         Data  : Object_Data_Access := Initialize( Port );
      end record;

end ASCL.SV.Server;



