--|
--| Filename        : $Source: /home/ACDE/ACDE/ASCL/RCS/template.adb,v $
--| Description     : User Object
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/23 09:48:34 $
--| Status          : $State: Released $
--| Version         : $Revision: 1.3 $
--|
--| Functional Description
--| ======================
--|
--|
--| Restrictions
--| ============
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: template.adb,v $
--| Revision 1.3  2000/01/23 09:48:34  ACDE
--| Release Version base_1_1
--|
--| Revision 1.2  2000/01/23 09:46:34  ACDE
--| Release Version 1_0
--|
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Unchecked_Deallocation;

with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
use  ASCL;

package body Template is

   Version : constant String := "$Id: template.adb,v 1.3 2000/01/23 09:48:34 ACDE Released ACDE $";
   Module  : constant String := "user.adb";

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         debug   : Debugging_Support.Handle := null;
      end record;

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object'Class;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Get pointer to object data
   --|
   function Instance( this : Object'Class; info : String ) 
                                               return  Object_Data_Access is
   begin
      if this.Data = null then
         raise Not_Initialized;
      end if;

      return this.Data;
   exception
      when The_Error: Others => 
         Error( this, The_Error, ".Instance");
	 raise;
   end Instance;
			
   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Initialize the Object data and call the Initialization
   --| procedure of the extension.
   --|
   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data  : Object_Data_Access renames this.data;
   begin
      Enter( debug, Module & ".Initialize" );
      if Data /= null then
         raise Usage_Error;
      end if;

      Data := new Object_Data;
      if Data = null then
         raise Out_Of_Memory;
      end if;

      Data.Debug := debug; 

      Leave( debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extension.
   --|
   procedure Finalize( this : in out Object'Class ) is
      data   : Object_Data_Access renames this.data;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      if data = null then
         return;
      end if;

      Free( data );

      this.data := null;

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| This function reads an attribute
   --|
   function Get( this : in Object'Class ) return Integer is
      data : Object_Data_Access := Instance( this, ".Get");
   begin
 
      return 1;

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Attribute");
         raise;

   end Get;

   --|
   --| Set some attribute
   --|
   procedure Set( this : in out Object'Class; value : in Integer ) is
      Data : Object_Data_Access := Instance( this, ".Set");
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Set " & Integer'Image(value) );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Leave( Debug, "** exception **");
         Error( this, The_Error, ".Action" );
         raise;
   end Set;

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Purpose:
   --|
   --| Pre-Conditions:
   --|
   --| Post-Conditions:
   --|
   --|
   procedure Perform( this : in out Object'Class; text : in String ) is
      Data  : Object_Data_Access := Instance( this, ".Perform" );
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Perform");

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Leave(Debug, "**exception**");
         Error( this, The_Error, ".Perform" );
         raise;
   end Perform;

end Template;

